@@ -42,6 +42,11 @@ char USR_FAVORITE_THREAD_FILE_PATH[1024];
 char EDITOR_CMD[1024]; 
 int FORCE_REFRESH = 0; 
 int THREAD_SORT_TYPE = NT_THREAD_SORT_BY_READ; 
+int INIT_DISP_STATE = NT_INTI_DISP_BOARDMENU; 
+int NT_MAINLOOP_POOLING_INTERVAL;
+int NT_PTHREAD_POOL_SIZE;
+int NT_PTHREAD_POOL_QUEUE_SIZE;
+
 static char *app_name = PACKAGE_NAME;
 static char *version_name = PACKAGE_VERSION;
 static char *def_editor_cmd = "vi";
@@ -69,6 +74,10 @@ int set_option(int argc, char* argv[])
 
 	MARU_ID = NULL;
 	MARU_PW = NULL;
+	
+	NT_MAINLOOP_POOLING_INTERVAL = 1000;
+	NT_PTHREAD_POOL_SIZE = 5;
+	NT_PTHREAD_POOL_QUEUE_SIZE = 20;
 
 	uid = getuid();
 	pw = getpwuid(uid);
@@ -139,7 +148,7 @@ extern BOOL read_resource(const char *path)
 {
 	FILE *fp;
 	char *cptr, *key, *val;
-	char *sort_type;
+	char *sort_type, *init_disp;
 	char buf[128];
 
 	fp = fopen(path, "r");
@@ -180,6 +189,18 @@ extern BOOL read_resource(const char *path)
 			free(sort_type);
 			continue;
 		}
+		key = strstr(buf, "show");
+		if(key){
+			val = cptr+1;
+			init_disp = nt_trim(val);
+			if(!init_disp)
+				break;
+			if(0 == strcmp(init_disp, "favorite")){
+				INIT_DISP_STATE = NT_INTI_DISP_FAVORITE;
+			}
+			free(init_disp);
+			continue;
+		}
 	}
 	fclose(fp);
 	return TRUE;
