@@ -35,8 +35,9 @@
 #include "net/nt_socket.h"
 #include "net/nt_cookie.h"
 #include "utils/file.h"
-#include "utils/nt_usr_db.h"
+#include "usr/usr_db_t.h"
 #include "utils/nt_mutex.h"
+#include "utils/nt_pthread.h"
 #include "usr/favorite_t.h"
 #include "_2ch/_2ch.h"
 #include "_2ch/maru_2ch.h"
@@ -47,24 +48,28 @@
 
 #define S_SIZE 	(1024)
 
-static int draw_title(WINDOW *wp, wchar_t *title, attr_t attr);
-static BOOL DoLoop(WINDOW *scrp, void *db_handle,
+static int draw_title(WINDOW *wp, const wchar_t *title, attr_t attr);
+static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 			nt_favorite_handle h_favorite, 
 			nt_favorite_grp_handle h_favorite_grp);
 static void print_error(WINDOW *wp, const wchar_t *msg);
-static BOOL set_sel_item(nt_2ch_model_tp modelp,
-			nt_2ch_selected_item_tp sel_itemp, 
-			nt_searched_thread_tp sel_threadp, const wchar_t** errorpp);
 
+static void _2ch_selected_item_free(void *ptr)
+{
+	nt_2ch_selected_item_release_ref(
+		(nt_2ch_selected_item_handle)ptr);
+
+}
 
 int main(int argc, char *argv[])
 {
 	int result = 1;
 	WINDOW *scrp;
-	void *usr_db_handle;
+	nt_usr_db_handle usr_db_handle;
 	nt_favorite_handle h_favorite;
 	nt_favorite_grp_handle h_favorite_grp;
 	nt_link_tp text_linkp, text2_linkp;
+	const char *err_msg;
 
 	if(0 != set_option(argc, argv)){
 		return 1;
@@ -100,17 +105,26 @@ int main(int argc, char *argv[])
 	}
 	
 	nt_mutex_lib_init();
+	
+	if(!nt_pthread_lib_init(
+			NT_PTHREAD_POOL_SIZE, 
+			NT_PTHREAD_POOL_QUEUE_SIZE, 
+			&err_msg)){
+		if(err_msg)
+			fputs(err_msg, stderr);
+		goto ERROR_TRAP;
+	}
 
 	h_favorite = nt_favorite_alloc(L"favorite");
 	if(!h_favorite){
-		return FALSE;
+		goto ERROR_TRAP;
 	}
 	h_favorite_grp = nt_favorite_grp_alloc(h_favorite, 
 				L"お気に入りのスレッド",
 				NT_FAVORITE_GRP_FLAG_FOLDER_OPEN);
 	if(!h_favorite_grp){
 		nt_favorite_release_ref(h_favorite);
-		return FALSE;
+		goto ERROR_TRAP;
 	}
 	
 	text_linkp = nt_read_text_file(USR_FAVORITE_BOARD_FILE_PATH);
@@ -159,19 +173,21 @@ ERROR_TRAP:
 	clear();
 
 
-	nt_2ch_model_free(app_2ch_modelp);
+	nt_2ch_model_release_ref(app_2ch_model);
 
 	endwin();
 
 	nt_usr_db_finish_lib(usr_db_handle);
 	
+	nt_pthread_lib_finish();
+	
 	nt_mutex_lib_finish();
 
 	return (result);
 }
 
 
-static BOOL DoLoop(WINDOW *scrp, void *db_handle,
+static BOOL DoLoop(WINDOW *scrp, nt_usr_db_handle db_handle,
 		nt_favorite_handle h_favorite, 
 		nt_favorite_grp_handle h_favorite_grp)
 {
@@ -183,27 +199,44 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 	nt_window_tp search_winp = NULL;
 	nt_window_tp favorite_winp = NULL;
 	char buf[256];
-	wchar_t wch, *title;
-	const wchar_t *status_msg = NULL;
+	wchar_t title_buf[128];
+	wchar_t wch;
+	const wchar_t *title;
+	const wchar_t *status_msg;
 	BOOL result = FALSE;
-	nt_write_data_tp writep = NULL;
-	nt_maru_2ch_tp marup = NULL;
+	nt_write_data_handle h_write_data;
+	nt_maru_2ch_tp marup;
 	nt_cookie_tp cookiep;
-	nt_link_tp linkp;
 	char *search;
-	nt_searched_thread_tp sel_threadp;
-	nt_2ch_selected_item_t sel_items;
-	nt_2ch_selected_item_t sel_items_tmp;
+	nt_searched_thread_handle h_searched_thread;
+	nt_2ch_selected_item_handle h_sel_items;
+	nt_2ch_selected_item_handle h_sel_items_tmp;
 	nt_favorite_board_handle h_favorite_board;
 	nt_favorite_thread_handle h_favorite_thread;
 	void *handle;
+	nt_pthread_result_t async_data;
+	nt_link_tp linkp;
 	
-	disp_state = DISP_STATE_BOARDMENU; 
+	state = DISP_STATE_BOARDMENU;
+	if(INIT_DISP_STATE == NT_INTI_DISP_FAVORITE){
+		disp_state = DISP_STATE_FAVORITE; 
+	}else{
+		disp_state = DISP_STATE_BOARDMENU; 
+	}
 	cookiep = nt_load_cookies(USR_COOKIE_PATH);
 
 	if(MARU_ID && MARU_PW) 
 		marup = nt_maru_2ch_alloc(MARU_ID, MARU_PW);
 
+	h_write_data = NULL;
+	marup = NULL;
+	status_msg = NULL;
+	h_searched_thread = NULL;
+	h_sel_items_tmp = NULL;
+	h_sel_items = nt_2ch_selected_item_alloc();
+	if(!h_sel_items)
+		return FALSE;
+
 	ch = NT_KEY_NONE;
 	bwinp = nt_disp_win_alloc(scrp, LINES-1, COLS, 1, 0, buf);
 	if(!bwinp)
@@ -215,6 +248,7 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 	if(!rwinp)
 		return FALSE;
 	
+	
 	keypad(stdscr, true);
 	while(1){
 		if(FORCE_REFRESH)
@@ -228,12 +262,12 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			draw_title(scrp, L"板一覧", WA_REVERSE);
 			if(!nt_disp_win_move(scrp, bwinp, LINES-1, COLS, 1, 0))
 				goto END_WHILE;
-			state = disp_board_menu(bwinp, app_2ch_modelp, &sel_items);
+			state = disp_board_menu(bwinp, app_2ch_model, h_sel_items);
 			if(DISP_CMD(state)){
 				if(DISP_CMD(state) == DISP_CMD_ADD_FAVORITE){
 					h_favorite_board = nt_favorite_board_alloc(
 						h_favorite,
-						sel_items.selected_boardp->name);
+						nt_2ch_selected_item_get_board_name(h_sel_items));
 					if(h_favorite_board){
 						nt_favorite_board_release_ref(h_favorite_board);
 						//favorite_dump(h_favorite);
@@ -245,8 +279,9 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			if(DISP_STATE_ERROR == state){
 				goto END_WHILE;
 			}else if(DISP_STATE_THREADTITLE == state){
-				if(nt_read_board(&sel_items)){
+				if(nt_read_board(h_sel_items)){
 					disp_state = state;
+					state = DISP_STATE_BOARDMENU;
 					if(twinp->data){
 						free_threadlist_ctx(twinp->data);
 						twinp->data = NULL;
@@ -272,18 +307,18 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			werase(twinp->wp);
 			twinp->key = ch;
 			twinp->status_msg = NULL;
-			title = sel_items.selected_boardp->name;
+			title = nt_2ch_selected_item_get_board_name(h_sel_items);
 			draw_title(scrp, title, WA_REVERSE);
 			if(!nt_disp_win_move(scrp, twinp, LINES-1, COLS, 1, 0))
 				goto END_WHILE;
-			state = disp_threadlist(twinp, &sel_items, db_handle);
+			state = disp_threadlist(twinp, state, h_sel_items, db_handle);
 			if(DISP_CMD(state)){
 				if(DISP_CMD(state) == DISP_CMD_ADD_FAVORITE){
 					h_favorite_thread = nt_favorite_thread_alloc(
 						h_favorite_grp,
-						sel_items.selected_threadp->file_name,
-						sel_items.selected_boardp->name,
-						sel_items.selected_threadp->name);
+						nt_2ch_selected_item_get_thread_dat_name(h_sel_items),
+						nt_2ch_selected_item_get_board_name(h_sel_items),
+						nt_2ch_selected_item_get_thread_title(h_sel_items));
 					if(h_favorite_thread){
 						nt_favorite_thread_release_ref(h_favorite_thread);
 						//favorite_dump(h_favorite);
@@ -301,33 +336,35 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 					ch = NT_KEY_NONE;
 					continue;
 			}else if(DISP_STATE_REFRESH == state){
-					if(nt_read_board(&sel_items)){
-						if(twinp->data){
-							free_threadlist_ctx(twinp->data);
-							twinp->data = NULL;
-						}
-						ch = NT_KEY_NONE;
-						status_msg = NT_INFO_REFRESH_BOARD_SUCCESS;
-						continue;
-					}else{
-						status_msg = NT_ERR_MSG_COUDLNOT_READ_BOARD;
-					}
-			}else if(DISP_STATE_RESLIST == state){
-					if(!nt_read_thread(&sel_items)){
-						wclear(scrp);
-						ch = NT_KEY_NONE;
-						continue;
-					}
+				if(nt_read_board(h_sel_items)){
 					if(twinp->data){
-						init_threadlist_ctx(twinp->data);
+						free_threadlist_ctx(twinp->data);
+						twinp->data = NULL;
 					}
-					if(rwinp->data){
-						free_reslist_ctx(rwinp->data);
-						rwinp->data = NULL;
-					}
-					disp_state = state;
+					ch = NT_KEY_NONE;
+					status_msg = NT_INFO_REFRESH_BOARD_SUCCESS;
+					continue;
+				}else{
+					status_msg = NT_ERR_MSG_COUDLNOT_READ_BOARD;
+				}
+				state = DISP_STATE_THREADTITLE;
+			}else if(DISP_STATE_RESLIST == state){
+				if(!nt_read_thread(h_sel_items)){
+					wclear(scrp);
 					ch = NT_KEY_NONE;
 					continue;
+				}
+				if(twinp->data){
+					init_threadlist_ctx(twinp->data);
+				}
+				if(rwinp->data){
+					free_reslist_ctx(rwinp->data);
+					rwinp->data = NULL;
+				}
+				disp_state = state;
+				state = DISP_STATE_THREADTITLE;
+				ch = NT_KEY_NONE;
+				continue;
 			}else if(DISP_STATE_SEARCH_THREAD == state){
 				state = DISP_STATE_THREADTITLE;
 				disp_state = DISP_STATE_SEARCH_THREAD;
@@ -344,29 +381,39 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			break;
 		case DISP_STATE_RESLIST:
 			werase(rwinp->wp);
-			title = sel_items.selected_threadp->name;
-			num = draw_title(scrp, title, WA_REVERSE);
+			title = nt_2ch_selected_item_get_thread_title(h_sel_items);
+			if(!title){
+				num = draw_title(scrp, L"Title not found.", WA_REVERSE);
+			}else if(-1 == swprintf(title_buf, sizeof(title_buf)/sizeof(wchar_t),
+					L"[%ls] - %ls", 
+					nt_2ch_selected_item_get_board_name(h_sel_items),
+					title)){
+				num = draw_title(scrp, title, WA_REVERSE);
+			}else{
+				num = draw_title(scrp, title_buf, WA_REVERSE);
+			}
 			if(!nt_disp_win_move(scrp, rwinp, 
 						LINES - num, COLS, num, 0))
 				goto END_WHILE;
 			rwinp->key = ch;
-			state = disp_reslist(rwinp, &sel_items, db_handle);
+			state = disp_reslist(rwinp, state, h_sel_items, db_handle);
 			if(DISP_STATE_ERROR == state){
 				goto END_WHILE;
 			}else if(state == DISP_STATE_REFRESH){
-					ch = NT_KEY_NONE;
-					if(!nt_read_thread(&sel_items)){
-						status_msg = NT_ERR_MSG_REFRESH_THREAD_FAILED;
-						wclear(scrp);
-						continue;
-					}
-					if(rwinp->data){
-						free_reslist_ctx(rwinp->data);
-						rwinp->data = NULL;
-					}
-					status_msg = NT_INFO_REFRESH_THREAD_SUCCESS;
+				state = DISP_STATE_RESLIST;
+				ch = NT_KEY_NONE;
+				if(!nt_read_thread(h_sel_items)){
+					status_msg = NT_ERR_MSG_REFRESH_THREAD_FAILED;
 					wclear(scrp);
 					continue;
+				}
+				if(rwinp->data){
+					free_reslist_ctx(rwinp->data);
+					rwinp->data = NULL;
+				}
+				status_msg = NT_INFO_REFRESH_THREAD_SUCCESS;
+				wclear(scrp);
+				continue;
 			}else if(state == DISP_STATE_SEARCH_THREAD){
 				state = DISP_STATE_RESLIST;
 				disp_state = DISP_STATE_SEARCH_THREAD;
@@ -385,38 +432,39 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			break;
 		case DISP_STATE_EDITOR:
 			disp_state = DISP_STATE_RESLIST;
+			state = DISP_STATE_RESLIST;
 			ch = NT_KEY_NONE;
-			if(writep)
-				nt_write_data_free(writep);
-			writep = nt_write_data_alloc();
-			if(!writep)
+			if(h_write_data)
+				nt_write_data_release_ref(h_write_data);
+			h_write_data = nt_write_data_alloc();
+			if(!h_write_data)
 				goto END_WHILE;
-			if(disp_editor(writep)){
+			if(disp_editor(h_write_data)){
 				if(marup && !marup->sid)
 					get_session_id(marup);
-				if(nt_write_msg(&sel_items, 
-						writep, cookiep, marup)){
+				if(nt_write_msg(h_sel_items, 
+						h_write_data, cookiep, marup)){
 					disp_state = DISP_STATE_HTML_RESULT;
 					continue;
 				}
 			}
-			status_msg = writep->status_msg;
+			status_msg = nt_write_data_get_status_msg(h_write_data);
 			wclear(scrp);
 			continue;
 		case DISP_STATE_HTML_RESULT:
-			nresult = disp_html_result(writep);
+			nresult = disp_html_result(h_write_data);
 			if(0 == nresult){
-				if(!nt_read_thread(&sel_items))
+				if(!nt_read_thread(h_sel_items))
 					break;
 				if(rwinp->data){
 					free_reslist_ctx(rwinp->data);
 					rwinp->data = NULL;
 				}
 			}else if(1 == nresult){
-				if(nt_write_msg(&sel_items, 
-						writep, cookiep, marup)){
-					if(0 == disp_html_result(writep)){
-						if(!nt_read_thread(&sel_items))
+				if(nt_write_msg(h_sel_items, 
+						h_write_data, cookiep, marup)){
+					if(0 == disp_html_result(h_write_data)){
+						if(!nt_read_thread(h_sel_items))
 							break;
 						if(rwinp->data){
 							free_reslist_ctx(rwinp->data);
@@ -426,8 +474,9 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 				}
 			}
 			disp_state = DISP_STATE_RESLIST;
+			state = DISP_STATE_RESLIST;
 			ch = NT_KEY_NONE;
-			status_msg = writep->status_msg;
+			status_msg = nt_write_data_get_status_msg(h_write_data);
 			wclear(scrp);
 			continue;
 		case DISP_STATE_SEARCH_THREAD:
@@ -442,19 +491,21 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			if(nt_get_search_text(buf, &search)){
 				if(search){
 					linkp = nt_search_all_board(
-							app_2ch_modelp, search, &status_msg);
+							app_2ch_model, search, &status_msg);
 					free(search);
 					buf[0] = '\0';
 				}
 			}
+			h_searched_thread = NULL;
 			disp_state = disp_thread_search(search_winp, 
-					state, linkp, &sel_threadp);
+					state, linkp, &h_searched_thread);
 			ch = NT_KEY_NONE;
-			if(sel_threadp){
-				if(set_sel_item(app_2ch_modelp,
-						&sel_items_tmp, sel_threadp, &status_msg)){
+			if(h_searched_thread){
+				if(nt_set_sel_item(app_2ch_model,
+						&h_sel_items_tmp, h_searched_thread, &status_msg)){
 					disp_state = DISP_STATE_RESLIST;
-					sel_items = sel_items_tmp;
+					nt_2ch_selected_item_release_ref(h_sel_items);
+					h_sel_items = h_sel_items_tmp;
 					if(rwinp->data){
 						free_reslist_ctx(rwinp->data);
 						rwinp->data = NULL;
@@ -463,8 +514,12 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 						free_threadlist_ctx(twinp->data);
 						twinp->data = NULL;
 					}
+					nt_searched_thread_release_ref(h_searched_thread);
+					h_searched_thread = NULL;
+					continue;
 				}
-				continue;
+				nt_searched_thread_release_ref(h_searched_thread);
+				h_searched_thread = NULL;
 			}
 			if(disp_state != DISP_STATE_SEARCH_THREAD){
 				continue;
@@ -478,8 +533,17 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 			if(!favorite_winp)
 				goto END_WHILE;
 			favorite_winp->key = ch;
-			state = disp_favorite(favorite_winp, state, h_favorite, 
-							&handle,  &sel_threadp);
+			h_searched_thread = NULL;
+			if(!favorite_winp->data){
+				linkp = nt_favorite_get_update_board_list(app_2ch_model, h_favorite);
+				if(linkp){
+					nt_read_board_list(linkp);
+					nt_all_link_free(linkp, _2ch_selected_item_free);
+				}
+			}
+			state = disp_favorite(favorite_winp, state, app_2ch_model,
+							h_favorite, db_handle, 
+							&handle,  &h_searched_thread);
 			if(DISP_CMD(state)){
 				ch = NT_KEY_NONE;
 				if(DISP_CMD(state) == DISP_CMD_DEL_FAVORITE_BOARD){
@@ -500,11 +564,14 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 					}
 				}else if(DISP_CMD(state) == DISP_CMD_SEL_FAVORITE_BOARD ||
 						DISP_CMD(state) == DISP_CMD_SEL_FAVORITE_THREAD){
-					if(set_sel_item(app_2ch_modelp,
-							&sel_items_tmp, sel_threadp, &status_msg)){
+					assert(h_searched_thread);
+					if(nt_set_sel_item(app_2ch_model,
+							&h_sel_items_tmp, h_searched_thread, &status_msg)){
 						disp_state = (DISP_CMD(state) == DISP_CMD_SEL_FAVORITE_BOARD)
 							? DISP_STATE_THREADTITLE : DISP_STATE_RESLIST;
-						sel_items = sel_items_tmp;
+						state = DISP_STATE_FAVORITE;
+						nt_2ch_selected_item_release_ref(h_sel_items);
+						h_sel_items = h_sel_items_tmp;
 						if(rwinp->data){
 							free_reslist_ctx(rwinp->data);
 							rwinp->data = NULL;
@@ -514,15 +581,22 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 							twinp->data = NULL;
 						}
 					}
-					if(sel_threadp->board_name)
-						free(sel_threadp->board_name);
-					if(sel_threadp->dat_name)
-						free(sel_threadp->dat_name);
-					free(sel_threadp);
+					nt_searched_thread_release_ref(h_searched_thread);
+					h_searched_thread = NULL;
 				}else{
 					disp_state = state = DISP_STATE_FAVORITE;
 				}
 				continue;
+			}else if(state == DISP_STATE_REFRESH){
+				state = DISP_STATE_FAVORITE;
+				ch = NT_KEY_NONE;
+				if(favorite_winp->data){
+					free_favorite_ctx(favorite_winp->data);
+					favorite_winp->data = NULL;
+				}
+				status_msg = NT_INFO_REFRESH_FAVORITE_SUCCESS;
+				wclear(scrp);
+				continue;
 			}else if(state != DISP_STATE_FAVORITE){
 				ch = NT_KEY_NONE;
 				disp_state = state;
@@ -543,7 +617,20 @@ static BOOL DoLoop(WINDOW *scrp, void *db_handle,
 		move(LINES-1,COLS-1);
 		touchwin(scrp);
 		wrefresh(scrp);
-		ch = getch();
+		
+		timeout(NT_MAINLOOP_POOLING_INTERVAL);
+		do{
+			ch = getch();
+			if(ch == ERR){
+				async_data = nt_pthread_get_result_from_que();
+				if(async_data.code == NT_PTHREAD_RESULT_UPDATE_BOARD){
+					
+					ch = NT_KEY_CMD_BOARD_UPDATE;
+				}
+			}
+		}while(ch == '\n' || ch == '\r' || ch == ERR);
+		timeout(-1);
+		
 		switch(ch){
 		case NT_KEY_UP:
 		case KEY_UP:
@@ -597,8 +684,11 @@ END_WHILE:
 		nt_unload_cookie(cookiep);
 	if(marup)
 		nt_maru_2ch_free(marup);
-	if(writep)
-		nt_write_data_free(writep);
+	if(h_write_data)
+		nt_write_data_release_ref(h_write_data);
+	if(h_searched_thread)
+		nt_searched_thread_release_ref(h_searched_thread);
+	nt_2ch_selected_item_release_ref(h_sel_items);
 	free_board_menu_ctx(bwinp->data);
 	free_threadlist_ctx(twinp->data);
 	free_reslist_ctx(rwinp->data);
@@ -619,58 +709,9 @@ END_WHILE:
 	return result;
 }
 
-static BOOL set_sel_item(nt_2ch_model_tp modelp,
-			nt_2ch_selected_item_tp sel_itemp, 
-			nt_searched_thread_tp sel_threadp, const wchar_t** errorpp)
-{
-	nt_category_tp categoryp;
-	nt_board_tp boardp;
-	nt_thread_tp threadp;
-	
-	assert(sel_threadp);
-	assert(sel_threadp->board_name);
-	//assert(sel_threadp->dat_name);
-	
-	boardp = nt_get_board_by_name(
-			modelp, sel_threadp->board_name, &categoryp);
-	if(!boardp || !categoryp){
-		*errorpp = L"板情報が見つかりませんでした";
-		return FALSE;
-	}
-		
-	sel_itemp->selected_categoryp = categoryp;
-	sel_itemp->selected_boardp = boardp;
-	sel_itemp->selected_threadp = NULL;
-	
-	if(!boardp->threadlistp){
-		if(!nt_read_board(sel_itemp) 
-				|| !boardp->threadlistp){
-			*errorpp = L"板が開けませんでした";
-			return FALSE;
-		}
-	}
-	if(!sel_threadp->dat_name)
-		return TRUE;
-	
-	threadp = nt_get_thread_by_dat_name(
-			boardp->threadlistp, sel_threadp->dat_name);
-	if(!threadp){
-		*errorpp = L"スレッド情報が見つかりませんでした";
-		return FALSE;
-	}
-	
-	sel_itemp->selected_threadp = threadp;
-	
-	if(!nt_read_thread(sel_itemp)){
-		*errorpp = L"スレッドが開けませんでした";
-		return FALSE;
-	}
-	
-	return TRUE;
-}
 
 
-static int draw_title(WINDOW *wp, wchar_t *title, attr_t attr)
+static int draw_title(WINDOW *wp, const wchar_t *title, attr_t attr)
 {
 	move(0,0);
 	int  num;
