@@ -23,12 +23,14 @@
 #include <ncursesw/ncurses.h>
 
 #include "env.h"
+#include "nt_string.h"
 #include "utils/nt_std_t.h"
 #include "utils/text.h"
 #include "_2ch/model_2ch.h"
 #include "_2ch/search_2ch.h"
 #include "ui/disp.h"
 #include "ui/disp_string.h"
+#include "usr/usr_db_t.h"
 
 
 typedef struct tag_favorite_ctx_t *favorite_ctx_tp;
@@ -48,9 +50,9 @@ static int get_screen_offset(
 		favorite_ctx_tp ctxp, nt_favorite_handle h_favorite, int cursor);
 
 int disp_favorite(nt_window_tp wp, 
-		int prev_state,
-		nt_favorite_handle h_favorite,
-		void **result_hadle, nt_searched_thread_tp *sel_threadpp)
+		int prev_state, nt_2ch_model_handle h_model,
+		nt_favorite_handle h_favorite, nt_usr_db_handle h_usr_db,
+		void **result_hadle, nt_searched_thread_handle *h_sel_threadp)
 {
 	favorite_ctx_tp ctxp;
 	attr_t attr;
@@ -66,27 +68,45 @@ int disp_favorite(nt_window_tp wp,
 	int result_state;
 	BOOL del_flag;
 	BOOL sel_flag;
+	BOOL sel_flag2;
 	int offset;
+	int read_count, num_res;
 	
 	result_state = DISP_STATE_FAVORITE;
 	del_flag = FALSE;
 	sel_flag = FALSE;
+	sel_flag2 = FALSE;
 	
 	ctxp = (favorite_ctx_tp)wp->data;
 	if(!ctxp){
 		ctxp = init_context();
 		if(!ctxp)
 			return DISP_STATE_ERROR;
-			
 		wp->data = ctxp;
 	}
-	if(prev_state != DISP_STATE_FAVORITE)
+	if(prev_state != DISP_STATE_FAVORITE){
+		if(!nt_favorite_set_read_count(h_favorite, h_usr_db)){
+			fputs("Couldn't open user_db handle", stderr);
+		}
 		ctxp->prev_state = prev_state;
+	}
 	
 	switch(wp->key){
+	case NT_KEY_CMD_BOARD_UPDATE:
+		if(!nt_favorite_set_num_res(h_model, h_favorite)){
+			fputs("Couldn't open user_db handle", stderr);
+		}
+		break;
+	case NT_KEY_REFRESH:
+		return DISP_STATE_REFRESH; 
 	case NT_KEY_CLOSE:
 	case KEY_LEFT:
+		return DISP_STATE_BOARDMENU;
+	case NT_KEY_RIGHT:
 		return ctxp->prev_state;
+	case NT_KEY_LEFT:
+		sel_flag2 = TRUE;
+		break;
 	case NT_KEY_UP:
 	case KEY_UP:
 		if(ctxp->cur_cursor > 0)
@@ -164,13 +184,11 @@ int disp_favorite(nt_window_tp wp,
 						*result_hadle = h_board;
 						result_state |= DISP_CMD_DEL_FAVORITE_BOARD;
 					}else if(sel_flag){
-						*sel_threadpp = malloc(sizeof(nt_searched_thread_t));
-						if(*sel_threadpp){
-							(*sel_threadpp)->board_name = nt_w_str_clone(cptr);
-							(*sel_threadpp)->dat_name = NULL;
-							(*sel_threadpp)->title = NULL;
+						board_name = nt_favorite_board_get_name(h_board);
+						(*h_sel_threadp) = nt_searched_thread_alloc(
+							board_name, NULL, NULL);
+						if((*h_sel_threadp))
 							result_state |= DISP_CMD_SEL_FAVORITE_BOARD;
-						}
 					}
 				}else{
 					attr = 0;
@@ -211,6 +229,8 @@ int disp_favorite(nt_window_tp wp,
 					(nt_favorite_thread_handle)nt_enum_fetch(h_enum_thread))){
 				board_name = nt_favorite_thread_get_board_name(h_thread);
 				dat_name = nt_favorite_thread_get_dat_name(h_thread);
+				read_count = nt_favorite_thread_get_read_count(h_thread);
+				num_res = nt_favorite_thread_get_estimate_res_count(h_thread);
 				title = nt_favorite_thread_get_title(h_thread);
 				if(cursor == ctxp->cur_cursor){
 					attr = WA_REVERSE;
@@ -218,18 +238,51 @@ int disp_favorite(nt_window_tp wp,
 						*result_hadle = h_thread;
 						result_state |= DISP_CMD_DEL_FAVORITE_THREAD;
 					}else if(sel_flag){
-						(*sel_threadpp) = malloc(sizeof(nt_searched_thread_t));
-						if(*sel_threadpp){
-							(*sel_threadpp)->board_name = nt_w_str_clone(board_name);
-							(*sel_threadpp)->dat_name = nt_w_str_clone(dat_name);
-							(*sel_threadpp)->title = NULL;
+						(*h_sel_threadp) = nt_searched_thread_alloc(
+							board_name, dat_name, NULL);
+						if((*h_sel_threadp))
 							result_state |= DISP_CMD_SEL_FAVORITE_THREAD;
-						}
+					}else if(sel_flag2){
+						(*h_sel_threadp) = nt_searched_thread_alloc(
+							board_name, NULL, NULL);
+						if((*h_sel_threadp))
+							result_state |= DISP_CMD_SEL_FAVORITE_BOARD;
 					}
 				}else{
 					attr = 0;
 				}
-				swprintf(wc, sizeof(wc), L"%ls[%ls]", title, board_name);
+				if(num_res > 0){
+					if(num_res > read_count){
+						if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
+							L"[%ls] - %ls (%d)未読%d", board_name, title, num_res, num_res-read_count)){
+							break;
+						}
+					}else if(read_count > 0){
+						if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
+							L"[%ls] - %ls (%d)未読0", board_name, title, num_res)){
+							break;
+						}
+					}else{
+						if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
+							L"[%ls] - %ls (%d)", board_name, title, num_res)){
+							break;
+						}
+					}
+				}else{
+					if(read_count > 0){
+						if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
+							L"%ls[%ls] - %ls (%d)", NT_TEXT_UNTRACKED_LOG_NAME,
+							board_name, title, read_count)){
+							break;
+						}
+					}else{
+						if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
+							L"%ls[%ls] - %ls", NT_TEXT_UNTRACKED_LOG_NAME,
+							board_name, title)){
+							break;
+						}
+					}
+				}
 				len = wcslen(wc);
 				if(row >= ctxp->scroll_pos){
 					wmove(wp->wp, row - ctxp->scroll_pos, 0);
