@@ -25,8 +25,9 @@
 #include <regex.h>
 
 #include	"env.h"
+#include	"error.h"
 #include	"utils/nt_std_t.h"
-#include	"utils/nt_usr_db.h"
+#include	"usr/usr_db_t.h"
 #include	"utils/text.h"
 #include	"_2ch/_2ch.h"
 #include	"_2ch/model_2ch.h"
@@ -45,6 +46,7 @@
 typedef struct tag_ctx_reslist_t *ctx_reslist_tp;
 typedef struct tag_ctx_reslist_t
 {
+	int prev_state;
 	int res_num;
 	int cur_res;
 	int cur_res_offset;
@@ -66,7 +68,8 @@ typedef struct tag_ctx_reslist_t
 
 typedef struct tag_res_data_t *res_data_tp;
 typedef struct tag_res_data_t {
-	nt_res_t *resp;
+	nt_res_handle h_res;
+	wchar_t *res_msg;
 	nt_link_tp cite_num_list;
 	nt_link_tp cited_num_list;
 	int msg_line_num;
@@ -75,9 +78,9 @@ typedef struct tag_res_data_t {
 	int id_num;
 }res_data_t;
 
-static ctx_reslist_tp init_context(nt_2ch_selected_item_tp selectp, 
-			void *usr_db_handle);
-static BOOL reslist_clone(nt_thread_tp threadp, ctx_reslist_tp ctxp);
+static ctx_reslist_tp init_context(nt_2ch_selected_item_handle h_select, 
+			nt_usr_db_handle usr_db_handle);
+static BOOL reslist_clone(nt_thread_handle h_thread, ctx_reslist_tp ctxp);
 static void int_ptr_free(void *ptr);
 static int parse_cmd1(const char *param, int *statep, const char **end);
 static BOOL search_line_asc(regex_t *regexp, nt_link_tp reslistp, 
@@ -101,8 +104,8 @@ static int get_id_num(ctx_reslist_tp ctxp, const wchar_t *misc);
 static BOOL set_res_header(ctx_reslist_tp ctxp, res_data_tp res_datap,
 				wchar_t *buf, size_t buf_len);
 
-int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
-	void *usr_db_handle)
+int disp_reslist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_select,
+	nt_usr_db_handle usr_db_handle)
 {
 	ctx_reslist_tp ctxp, child_ctxp;
 	res_data_tp res_datap;
@@ -130,7 +133,7 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 
 	ctxp = (ctx_reslist_tp)wp->data;
 	if(!ctxp){
-		ctxp = init_context(selectp, usr_db_handle);
+		ctxp = init_context(h_select, usr_db_handle);
 		if(!ctxp)
 			return DISP_STATE_ERROR;
 		wp->data = ctxp;
@@ -142,6 +145,10 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			ctxp = (ctx_reslist_tp)wp->data;
 		}
 	}
+	
+	if(prev_state != DISP_STATE_RESLIST)
+		ctxp->prev_state = prev_state;
+
 	ch = wp->key;
 
 
@@ -157,10 +164,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			}
 
 			if(res_datap->msg_header_line_num <= 0){
-				/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-						L"%5d. %ls %ls %ls", 
-						res_datap->resp->seq_no, res_datap->resp->name,
-						res_datap->resp->mail, res_datap->resp->misc)){*/
 				if(!set_res_header((ctx_reslist_tp)wp->data, 
 						res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 					res_datap->msg_header_line_num = 1;
@@ -214,7 +217,7 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 		return DISP_STATE_REFRESH; 
 	case NT_KEY_CLOSE:
 	case KEY_LEFT:
-		return DISP_STATE_THREADTITLE; 
+		return ctxp->prev_state; 
 	case NT_KEY_BOTTOM:
 	case KEY_END:
 		if(ctxp->cur_res == ctxp->res_num - 1)
@@ -336,10 +339,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 						parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
 					}
 					if(res_datap->msg_header_line_num <= 0){
-						/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-								L"%5d. %ls %ls %ls", 
-								res_datap->resp->seq_no, res_datap->resp->name,
-								res_datap->resp->mail, res_datap->resp->misc)){*/
 						if(!set_res_header((ctx_reslist_tp)wp->data, 
 								res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 							res_datap->msg_header_line_num = 1;
@@ -482,10 +481,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
 		}
 		if(res_datap->msg_header_line_num <= 0){
-			/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-					L"%5d. %ls %ls %ls", 
-					res_datap->resp->seq_no, res_datap->resp->name,
-					res_datap->resp->mail, res_datap->resp->misc)){*/
 			if(!set_res_header((ctx_reslist_tp)wp->data, 
 						res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 				res_datap->msg_header_line_num = 1;
@@ -523,10 +518,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
 		}
 		if(res_datap->msg_header_line_num <= 0){
-			/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-					L"%5d. %ls %ls %ls", 
-					res_datap->resp->seq_no, res_datap->resp->name,
-					res_datap->resp->mail, res_datap->resp->misc)){*/
 			if(!set_res_header((ctx_reslist_tp)wp->data, 
 					res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 				res_datap->msg_header_line_num = 1;
@@ -574,10 +565,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
 			}
 			if(res_datap->msg_header_line_num <= 0){
-				/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-						L"%5d. %ls %ls %ls", 
-						res_datap->resp->seq_no, res_datap->resp->name,
-						res_datap->resp->mail, res_datap->resp->misc)){*/
 				if(!set_res_header((ctx_reslist_tp)wp->data, 
 						res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 					res_datap->msg_header_line_num = 1;
@@ -631,10 +618,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 				parse_res_msg(ctxp->res_disp_list, res_datap, wp->cols-5);
 			}
 			if(res_datap->msg_header_line_num <= 0){
-				/*if(-1 == swprintf(buf, sizeof(buf)-1, 
-						L"%5d. %ls %ls %ls", 
-						res_datap->resp->seq_no, res_datap->resp->name,
-						res_datap->resp->mail, res_datap->resp->misc)){*/
 				if(!set_res_header((ctx_reslist_tp)wp->data, 
 						res_datap, buf, sizeof(buf)/sizeof(buf[0]))){
 					res_datap->msg_header_line_num = 1;
@@ -798,8 +781,8 @@ static int get_id_num(ctx_reslist_tp ctxp, const wchar_t *misc)
 	int num;
 	num = 0;
 	res_data_tp datap;
-	nt_res_tp resp;
 	wchar_t *cptr;
+	const wchar_t *misc2;
 	nt_link_tp linkp, disp_list;
 	
 	assert(ctxp);
@@ -816,8 +799,8 @@ static int get_id_num(ctx_reslist_tp ctxp, const wchar_t *misc)
 	linkp = disp_list;
 	do{
 		datap = (res_data_tp)linkp->data;
-		resp = datap->resp;
-		if(wcsstr(resp->misc, cptr))
+		misc2 = nt_res_get_misc(datap->h_res);
+		if(wcsstr(misc2, cptr))
 			num++;
 		linkp = linkp->next;
 	}while(linkp != disp_list);
@@ -829,24 +812,27 @@ static int get_id_num(ctx_reslist_tp ctxp, const wchar_t *misc)
 static BOOL set_res_header(ctx_reslist_tp ctxp, res_data_tp res_datap,
 				wchar_t *buf, size_t buf_len)
 {
-	int id_num;
+	const wchar_t *name, *mail, *misc;
+	int seq_no, id_num;
+	
+	seq_no = nt_res_get_seq_number(res_datap->h_res);
+	name = nt_res_get_name(res_datap->h_res);
+	mail = nt_res_get_mail(res_datap->h_res);
+	misc = nt_res_get_misc(res_datap->h_res);
+	
 	id_num = res_datap->id_num;
 	if(id_num < 0){
-		id_num = get_id_num(ctxp, res_datap->resp->misc);
+		id_num = get_id_num(ctxp, misc);
 		res_datap->id_num = id_num;
 	}
 	if(id_num <= 1){
 		if(-1 == swprintf(buf, buf_len-1, 
-				L"%5d. %ls %ls %ls", 
-				res_datap->resp->seq_no, res_datap->resp->name,
-				res_datap->resp->mail, res_datap->resp->misc)){
+				L"%5d. %ls %ls %ls", seq_no, name, mail, misc)){
 			return FALSE;
 		}
 	}else{
 		if(-1 == swprintf(buf, buf_len-1, 
-				L"%5d. %ls %ls %ls(%d)", 
-				res_datap->resp->seq_no, res_datap->resp->name,
-				res_datap->resp->mail, res_datap->resp->misc, id_num)){
+				L"%5d. %ls %ls %ls(%d)", seq_no, name, mail, misc, id_num)){
 			return FALSE;
 		}
 	}
@@ -894,22 +880,28 @@ static int parse_cmd1(const char *param, int *statep, const char **end)
 	return NT_CMD_NONE;
 }
 
-static ctx_reslist_tp init_context(nt_2ch_selected_item_tp selectp,
-	void *usr_db_handle)
+static ctx_reslist_tp init_context(nt_2ch_selected_item_handle h_select,
+	nt_usr_db_handle usr_db_handle)
 {
+	nt_thread_handle h_thread;
 	ctx_reslist_tp ctxp;
-	nt_board_tp boardp;
-	nt_thread_tp threadp;
-
-	boardp = selectp->selected_boardp;
-	threadp = selectp->selected_threadp;
-	assert(boardp && threadp);
+	const wchar_t *board_name, *dat_name;
+	BOOL b_result;
+	
+	board_name = nt_2ch_selected_item_get_board_name(h_select);
+	dat_name = nt_2ch_selected_item_get_thread_dat_name(h_select);
+	assert(board_name && dat_name);
 
 	ctxp = (ctx_reslist_tp)calloc(1,sizeof(ctx_reslist_t));
 
 	if(!ctxp)
 		return NULL;
-
+	
+	h_thread = nt_get_selected_thread(h_select);
+	if(!h_thread){
+		free(ctxp);
+		return NULL;
+	}
 	ctxp->cur_res = 0;
 	ctxp->cur_res_offset = 0;
 	ctxp->max_cur_res = -1;
@@ -919,15 +911,18 @@ static ctx_reslist_tp init_context(nt_2ch_selected_item_tp selectp,
 	ctxp->sel_res_no = -1;
 	ctxp->sel_res_line = -1;
 	ctxp->child_ctx_stackp = NULL;
+	ctxp->prev_state = DISP_STATE_THREADTITLE;
 
-	if(!reslist_clone(threadp, ctxp)){
+	b_result = reslist_clone(h_thread, ctxp);
+	nt_thread_release_ref(h_thread);
+	if(!b_result){
 		free_reslist_ctx(ctxp);
 		return NULL;
 	}
 	
 	ctxp->prev_read_cnt = 
-		nt_usr_db_update_read_count(usr_db_handle,
-				boardp->name, threadp->file_name,
+		nt_usr_db_update_read_count(usr_db_handle, 
+				board_name, dat_name,
 				ctxp->res_num);
 	if(ctxp->prev_read_cnt < 0)
 		ctxp->prev_read_cnt = 0;
@@ -955,6 +950,7 @@ static ctx_reslist_tp init_sub_context(ctx_reslist_tp ctxp, nt_link_tp num_listp
 	child_ctxp->sel_res_line = -1;
 	child_ctxp->child_ctx_stackp = NULL;
 	child_ctxp->prev_read_cnt = 0;
+	child_ctxp->prev_state = ctxp->prev_state;
 
 	if(!reslist_copy(ctxp, child_ctxp, num_listp)){
 		nt_stack_free(child_ctxp->selected_num_stackp, NULL);
@@ -978,50 +974,43 @@ static ctx_reslist_tp init_sub_context(ctx_reslist_tp ctxp, nt_link_tp num_listp
 	return child_ctxp;
 }
 
-static BOOL reslist_clone(nt_thread_tp threadp, ctx_reslist_tp ctxp)
+static BOOL reslist_clone(nt_thread_handle h_thread, ctx_reslist_tp ctxp)
 {
-	nt_link_tp linkp, r_linkp;
-	nt_res_tp resp;
+	nt_enum_handle h_enum_res;
+	nt_link_tp linkp;
+	nt_res_handle h_res;
 	res_data_tp datap;
 	int num_res;
 	
-	assert(threadp);
-	assert(threadp->reslistp);
+	h_enum_res = nt_thread_get_res_enum(h_thread);
+	if(!h_enum_res)
+		return FALSE;
 	
 	num_res = 0;
-	linkp = threadp->reslistp;
-	do{
+	while(NULL != (h_res = (nt_res_handle)nt_enum_fetch(h_enum_res))){
 		datap = (res_data_tp)calloc(1, sizeof(res_data_t));
 		if(datap){
 			datap->id_num = -1;
-			resp = (nt_res_t *)linkp->data;
-			datap->resp = malloc(sizeof(nt_res_t));
-			if(datap->resp){
-				datap->resp->seq_no = resp->seq_no;
-				datap->resp->name = nt_w_str_clone(resp->name);
-				datap->resp->mail = nt_w_str_clone(resp->mail);
-				datap->resp->misc = nt_w_str_clone(resp->misc);
-				datap->resp->msg = nt_parse_res_msg(resp->msg,
+			datap->h_res = h_res;
+			datap->res_msg = nt_parse_res_msg(
+					nt_res_get_msg(h_res),
 					&datap->cite_num_list);
-				if(!datap->resp->name || !datap->resp->mail ||
-						!datap->resp->misc || !datap->resp->msg){
-					if(datap->resp->name)free(datap->resp->name);
-					if(datap->resp->mail)free(datap->resp->mail);
-					if(datap->resp->misc)free(datap->resp->misc);
-					if(datap->resp->msg)free(datap->resp->msg);
-					free(datap->resp);
-					free(datap);
-				}else{
-					r_linkp = nt_link_add_data(ctxp->res_disp_list, datap);
-					if(!ctxp->res_disp_list)
-						ctxp->res_disp_list = r_linkp;
-					num_res++;
-				}
+			if(!datap->res_msg){
+				datap->res_msg = nt_w_str_clone(NT_ERR_MSG_DECODE_TEXT);
 			}
+			linkp = nt_link_add_data(ctxp->res_disp_list, datap);
+			if(!linkp){
+				free(datap->res_msg);
+				free(datap);
+				continue;
+			}
+			if(!ctxp->res_disp_list)
+				ctxp->res_disp_list = linkp;
+			nt_res_add_ref(h_res);
+			num_res++;
 		}
-		linkp = linkp->next;
-	}while(linkp != threadp->reslistp);
-	
+	}
+	nt_enum_unset(h_enum_res);
 	ctxp->res_num = num_res;
 	return TRUE;
 }
@@ -1030,7 +1019,7 @@ static BOOL reslist_copy(ctx_reslist_tp ctxp, ctx_reslist_tp copy_ctxp, nt_link_
 {
 	nt_link_tp linkp, r_linkp;
 	nt_link_tp d_linkp;
-	nt_res_tp resp;
+	nt_res_handle h_res;
 	res_data_tp datap;
 	int num_res;
 	int seq_no;
@@ -1044,8 +1033,8 @@ static BOOL reslist_copy(ctx_reslist_tp ctxp, ctx_reslist_tp copy_ctxp, nt_link_
 		d_linkp = ctxp->res_disp_list;
 		do{
 			datap = (res_data_tp)d_linkp->data;
-			resp = datap->resp;
-			if(seq_no == resp->seq_no){
+			h_res = datap->h_res;
+			if(seq_no == nt_res_get_seq_number(h_res)){
 				r_linkp = nt_link_add_data(copy_ctxp->res_disp_list, datap);
 				if(r_linkp){
 					if(!copy_ctxp->res_disp_list)
@@ -1105,19 +1094,14 @@ static void res_msg_free(void *ptr)
 {
 	nt_link_tp curp, nextp;
 	res_data_tp res_datap;
-	nt_res_tp resp;
 	
 	assert(ptr);
 	
 	res_datap = (res_data_tp)ptr;
 	
-	resp = res_datap->resp;
+	free(res_datap->res_msg);
 	
-	free(resp->name);
-	free(resp->mail);
-	free(resp->misc);
-	free(resp->msg);
-	free(resp);
+	nt_res_release_ref(res_datap->h_res);
 	
 	if(res_datap->cite_num_list)
 		nt_all_link_free(res_datap->cite_num_list, NULL);
@@ -1161,7 +1145,8 @@ static nt_link_tp get_cited_num_list(nt_link_tp res_listp, int seq_no)
 			do{
 				if(wrkp->n_data == seq_no){
 					wrk2p = nt_link_add_n_data(
-							num_listp, res_datap->resp->seq_no);
+							num_listp, 
+							nt_res_get_seq_number(res_datap->h_res));
 					if(wrk2p && !num_listp)
 						num_listp = wrk2p;
 					break;
@@ -1189,13 +1174,13 @@ static int parse_res_msg(nt_link_tp disp_res_list,
 	const int delta = 64;
 
 	assert(res_datap);
-	assert(res_datap->resp);
-	assert(res_datap->resp->msg);
+	assert(res_datap->h_res);
+	assert(res_datap->res_msg);
 
 	//res_msg_free(res_datap);
 	res_datap->msg_line_num = 0;
 
-	len = wcslen(res_datap->resp->msg);
+	len = wcslen(res_datap->res_msg);
 	if(len == 0)
 		return 0;
 	buf_size = len + delta;
@@ -1207,7 +1192,7 @@ static int parse_res_msg(nt_link_tp disp_res_list,
 		return 0;
 	buf_idx = 0;
 
-	srcp = res_datap->resp->msg;
+	srcp = res_datap->res_msg;
 
 	lines = 0;
 	start = 0;
@@ -1288,7 +1273,7 @@ END_FOR:
 	if(!res_datap->cited_num_list){
 		res_datap->cited_num_list = 
 			get_cited_num_list(disp_res_list, 
-				res_datap->resp->seq_no);
+				nt_res_get_seq_number(res_datap->h_res));
 	}
 	return lines;
 }
@@ -1481,10 +1466,12 @@ static nt_link_tp parse_id_list(ctx_reslist_tp ctxp, const char *param)
 	char buf[64];
 	wchar_t wc[64];
 	wchar_t *cptr;
+	const wchar_t *misc;
 	int len;
 	const char *start, *end;
 	res_data_tp res_datap;
-	nt_res_tp resp;
+	nt_res_handle h_res;
+	int seq_no;
 	
 	assert(ctxp);
 	assert(ctxp->res_disp_list);
@@ -1498,7 +1485,7 @@ static nt_link_tp parse_id_list(ctx_reslist_tp ctxp, const char *param)
 	strcpy(buf,"ID:");
 	strncat(buf, start, len);
 	
-	if((size_t)-1 == mbstowcs(wc, buf, sizeof(wc)-1))
+	if((size_t)-1 == mbstowcs(wc, buf, sizeof(wc)/sizeof(wchar_t)-1))
 		return NULL;
 		
 	num_linkp = NULL;
@@ -1506,10 +1493,12 @@ static nt_link_tp parse_id_list(ctx_reslist_tp ctxp, const char *param)
 	linkp = disp_list;
 	do{
 		res_datap = (res_data_tp)linkp->data;
-		resp = res_datap->resp;
-		cptr = wcsstr(resp->misc, wc);
+		h_res = res_datap->h_res;
+		misc = nt_res_get_misc(h_res);
+		cptr = wcsstr(misc, wc);
 		if(cptr){
-			wrkp = nt_link_add_n_data(num_linkp, resp->seq_no);
+			seq_no = nt_res_get_seq_number(h_res);
+			wrkp = nt_link_add_n_data(num_linkp, seq_no);
 			if(!num_linkp)
 				num_linkp = wrkp;
 		}
@@ -1527,7 +1516,7 @@ static nt_link_tp parse_tree_list(ctx_reslist_tp ctxp, const char *param)
 	int len;
 	const char *start, *end;
 	res_data_tp res_datap;
-	nt_res_tp resp;
+	nt_res_handle h_res;
 	int num;
 	
 	assert(ctxp);
@@ -1550,8 +1539,8 @@ static nt_link_tp parse_tree_list(ctx_reslist_tp ctxp, const char *param)
 	linkp = disp_list;
 	do{
 		res_datap = (res_data_tp)linkp->data;
-		resp = res_datap->resp;
-		if(num == resp->seq_no){
+		h_res = res_datap->h_res;
+		if(num == nt_res_get_seq_number(h_res)){
 			if(res_datap->cite_num_list){
 				wrkp = res_datap->cite_num_list;
 				do{
@@ -1581,13 +1570,13 @@ static void search_up_tree(int seq_no, nt_link_tp disp_list, nt_link_tp *num_lin
 	nt_link_tp  linkp, wrkp;
 	nt_link_add_n_data(*num_linkp, seq_no);
 	res_data_tp res_datap;
-	nt_res_tp resp;
+	nt_res_handle h_res;
 	
 	linkp = disp_list;
 	do{
 		res_datap = (res_data_tp)linkp->data;
-		resp = res_datap->resp;
-		if(seq_no == resp->seq_no){
+		h_res = res_datap->h_res;
+		if(seq_no == nt_res_get_seq_number(h_res)){
 			if(res_datap->cite_num_list){
 				wrkp = res_datap->cite_num_list;
 				do{
@@ -1607,13 +1596,13 @@ static void search_down_tree(int seq_no, nt_link_tp disp_list, nt_link_tp *num_l
 	nt_link_tp  linkp, wrkp;
 	nt_link_add_n_data(*num_linkp, seq_no);
 	res_data_tp res_datap;
-	nt_res_tp resp;
+	nt_res_handle h_res;
 	
 	linkp = disp_list;
 	do{
 		res_datap = (res_data_tp)linkp->data;
-		resp = res_datap->resp;
-		if(seq_no == resp->seq_no){
+		h_res = res_datap->h_res;
+		if(seq_no == nt_res_get_seq_number(h_res)){
 			if(res_datap->cited_num_list){
 				wrkp = res_datap->cited_num_list;
 				do{
