@@ -27,7 +27,7 @@
 #include "error.h"
 #include "nt_string.h"
 #include "utils/nt_std_t.h"
-#include "utils/nt_usr_db.h"
+#include "usr/usr_db_t.h"
 #include "_2ch/_2ch.h"
 #include "_2ch/model_2ch.h"
 #include "_2ch/parse_2ch.h"
@@ -38,6 +38,7 @@
 typedef struct tag_ctx_threadlist_t *ctx_threadlist_tp;
 typedef struct tag_ctx_threadlist_t
 {
+	int prev_state;
 	int thread_num;
 	int tracked_thread_num;
 	int cursor_pos;
@@ -55,6 +56,20 @@ typedef struct tag_ctx_threadlist_t
 
 } ctx_threadlist_t;
 
+typedef struct tag_thread_disp_data_t{
+	int seq_no;
+	int num_read;
+	int num_res;
+	wchar_t *dat_name;
+	wchar_t *title;
+}thread_disp_data_t, *thread_disp_data_tp;
+
+static thread_disp_data_tp disp_data_alloc(
+		int seq_no, int num_res, const wchar_t *title, const wchar_t *dat_name);
+static void disp_data_free(void *ptr);
+static BOOL get_thread_data_by_dat_name(const wchar_t *dat_name, nt_link_tp disp_list);
+
+
 #define NT_CMD_ERR -1
 #define NT_CMD_NONE 0
 #define NT_CMD_SORT_NUMBER 1
@@ -65,14 +80,15 @@ typedef struct tag_ctx_threadlist_t
 #define NT_CMD_FAVORITE 6
 
 static int parse_cmd1(const char *param);
+static thread_disp_data_tp get_thread_by_seq_no(nt_link_tp linkp, int seq_no);
 static int get_thread_index_by_seqno(nt_link_tp linkp, int seq_no);
 static BOOL search_line_asc(regex_t *regexp, 
 				nt_link_tp threadlistp, int *sel_thread_no);
 static BOOL search_line_desc(regex_t *regexp, 
 				nt_link_tp threadlistp, int *sel_thread_no);
-static ctx_threadlist_tp init_context(nt_2ch_selected_item_tp selectp);
-static int merge_untracked_thread_log(nt_board_tp boardp,
-		nt_link_tp *disp_thread_listpp, nt_link_tp log_list);
+static ctx_threadlist_tp init_context(nt_2ch_selected_item_handle h_select);
+static int merge_untracked_thread_log(int init_seq_no, nt_board_handle h_board,
+		nt_link_tp disp_thread_listpp, nt_link_tp log_list);
 
 
 static int sort_by_number(void *lhs, void *rhs);
@@ -96,12 +112,14 @@ static void sort_thread_title(int sort_type, ctx_threadlist_tp ctxp)
 	}
 }
 
-int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
-				 void *db_handle)
+int disp_threadlist(nt_window_tp wp, int prev_state, nt_2ch_selected_item_handle h_select,
+				 nt_usr_db_handle db_handle)
 {
 	ctx_threadlist_tp ctxp;
-	nt_board_tp boardp;
-	nt_thread_tp threadp;
+	nt_board_handle h_board;
+	nt_thread_handle h_thread;
+	thread_disp_data_tp threadp;
+	const wchar_t *board_name;
 	nt_link_tp clistp, linkp;
 	int i, rows, nwrite, len, ch;
 	int btm_cur, num, wlines;
@@ -120,23 +138,29 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 
 	ctxp = (ctx_threadlist_tp)wp->data;
 	if(!ctxp){
-		ctxp = init_context(selectp);
+		ctxp = init_context(h_select);
 		if(!ctxp)
 			return DISP_STATE_ERROR;
 		wp->data = ctxp;
 		sort = TRUE;
 	}
+	if(prev_state != DISP_STATE_THREADTITLE)
+		ctxp->prev_state = prev_state;
 
-	boardp = selectp->selected_boardp;
+	h_board = nt_get_selected_board(h_select);
+	assert(h_board);
+	board_name = nt_board_get_name(h_board);
 
 	if(!ctxp->thread_data_list){
 		ctxp->thread_data_list =
 			nt_usr_db_query_read_count_list(
-					db_handle, boardp->name);
+					db_handle, board_name);
 		if(ctxp->thread_data_list){
 			ctxp->thread_num = 
-					merge_untracked_thread_log(boardp,
-							&ctxp->thread_disp_list,
+					merge_untracked_thread_log(
+							ctxp->tracked_thread_num+1,
+							h_board,
+							ctxp->thread_disp_list,
 							ctxp->thread_data_list);
 		}
 	}
@@ -149,10 +173,12 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 
 	switch(ch){
 	case NT_KEY_REFRESH: 
-		return DISP_STATE_REFRESH;
+		result_state = DISP_STATE_REFRESH;
+		goto ERROR_TRAP;
 	case NT_KEY_CLOSE:
 	case KEY_LEFT:
-		return DISP_STATE_BOARDMENU;
+		result_state = ctxp->prev_state;
+		goto ERROR_TRAP;
 	case NT_KEY_BOTTOM:
 	case KEY_END:
 		ctxp->cursor_pos = ctxp->thread_num - 1;
@@ -206,15 +232,18 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			case NT_CMD_SORT_NUMBER:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_NUMBER;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				return DISP_STATE_REFRESH;
+				result_state = DISP_STATE_REFRESH;
+				goto ERROR_TRAP;
 			case NT_CMD_SORT_READ:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_READ;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				return DISP_STATE_REFRESH;
+				result_state = DISP_STATE_REFRESH;
+				goto ERROR_TRAP;
 			case NT_CMD_SORT_UNREAD:
 				THREAD_SORT_TYPE = NT_THREAD_SORT_BY_UNREAD;
 				sort_thread_title(THREAD_SORT_TYPE, ctxp);
-				return DISP_STATE_REFRESH;
+				result_state = DISP_STATE_REFRESH;
+				goto ERROR_TRAP;
 			case NT_CMD_DEL_THREAD_LOG: 
 				if(!nt_strtok(wp->cmd_param,' ', &start, &end))
 					break;
@@ -224,9 +253,10 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 				if(0 == strcmp("*", cptr)){
 					free(cptr);
 					if(!nt_usr_db_delete_board_log(db_handle,
-							boardp->name))
+							board_name))
 						break;
-					return DISP_STATE_REFRESH;
+					result_state = DISP_STATE_REFRESH;
+					goto ERROR_TRAP;
 				}
 				linkp = nt_parse_number_list(cptr, NULL);
 				free(cptr);
@@ -235,26 +265,29 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 				clistp = linkp;
 				num = -1;
 				do{
-					threadp = nt_get_thread_by_seq_no(
+					threadp = get_thread_by_seq_no(
 						ctxp->thread_disp_list, linkp->n_data);
 					if(!threadp)
 						break;
 						
 					if(linkp->n_data != num){
 						nt_usr_db_delete_thread_log(db_handle,
-								boardp->name, 
-								threadp->file_name,
+								board_name, 
+								threadp->dat_name,
 								&ctxp->thread_data_list);
 						num = linkp->n_data;
 					}
 					linkp = linkp->next;
 				}while(clistp != linkp);
 				nt_all_link_free(linkp, NULL);
-				return DISP_STATE_REFRESH;
+				result_state = DISP_STATE_REFRESH;
+				goto ERROR_TRAP;
 			case NT_CMD_SEARCH_THREAD:
-				return DISP_STATE_SEARCH_THREAD;
+				result_state = DISP_STATE_SEARCH_THREAD;
+				goto ERROR_TRAP;
 			case NT_CMD_FAVORITE:
-				return DISP_STATE_FAVORITE;
+				result_state = DISP_STATE_FAVORITE;
+				goto ERROR_TRAP;
 			default:
 				break;
 			}
@@ -351,14 +384,24 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 				ctxp->cursor_pos);
 		if(!threadp)
 			break;
-		nt_set_selected_thread(selectp, threadp);
-		return DISP_STATE_RESLIST;
+		h_thread = nt_thread_dummy_alloc(
+				threadp->title, threadp->dat_name, threadp->num_res);
+		if(!h_thread)
+			break;
+		nt_set_selected_thread(h_select, h_thread);
+		nt_thread_release_ref(h_thread);
+		result_state = DISP_STATE_RESLIST;
+		goto ERROR_TRAP;
 	case NT_KEY_ADD:
 		threadp = nt_link_get_by_index(ctxp->thread_disp_list,
 				ctxp->cursor_pos);
 		if(!threadp)
 			break;
-		nt_set_selected_thread(selectp, threadp);
+		h_thread = nt_thread_dummy_alloc(
+				threadp->title, threadp->dat_name, threadp->num_res);
+		if(!h_thread)
+			break;
+		nt_set_selected_thread(h_select, h_thread);
 		result_state |= DISP_CMD_ADD_FAVORITE;
 	}
 
@@ -371,20 +414,20 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			attr = WA_BOLD;
 		else
 			attr = 0;
-		threadp = (nt_thread_tp)clistp->data;
-		len = wcslen(threadp->name);
+		threadp = (thread_disp_data_tp)clistp->data;
+		len = wcslen(threadp->title);
 		if(rows >= ctxp->scroll_pos){
 			wmove(wp->wp, rows - ctxp->scroll_pos, 0);
-			if(-1 == swprintf(buf, sizeof(buf)-1, 
+			if(-1 == swprintf(buf, sizeof(buf)/sizeof(wchar_t)-1, 
 							L"%5d.", threadp->seq_no))
 				continue;
 
 			nt_add_wstr(wp->wp, buf, attr);
-			nwrite = nt_add_wnstr(wp->wp, threadp->name, attr, 
+			nwrite = nt_add_wnstr(wp->wp, threadp->title, attr, 
 						wp->cols - 5); 
 		}else{
 			nwrite = nt_get_wc_count_within_colmns(
-					threadp->name, wp->cols - 5);
+					threadp->title, wp->cols - 5);
 		}
 		rows++;
 		if(rows >= (wp->lines+ctxp->scroll_pos))
@@ -395,12 +438,12 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			wmove(wp->wp, rows - ctxp->scroll_pos, 5);
 			if(nwrite < len){
 				nwrite = nt_add_wnstr(wp->wp, 
-					threadp->name + nwrite, 
+					threadp->title + nwrite, 
 					WA_UNDERLINE | attr, wp->cols - 5); 
 			}
 			if(ctxp->thread_data_list){
 				read_count = nt_usr_db_get_read_count_by_dat_name(
-						ctxp->thread_data_list, threadp->file_name);
+						ctxp->thread_data_list, threadp->dat_name);
 				if(read_count < 0)
 					read_count = 0;
 			}else{
@@ -409,7 +452,7 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 			buf[0] = L'\0';
 			if(read_count == 0){
 				if(-1 < swprintf(
-					buf, sizeof(buf)-1, L"(%d)  ", threadp->num_res)){
+					buf, sizeof(buf)/sizeof(wchar_t)-1, L"(%d)  ", threadp->num_res)){
 				}
 			}else{
 				if(-1 < swprintf(
@@ -430,62 +473,149 @@ int disp_threadlist(nt_window_tp wp, nt_2ch_selected_item_tp selectp,
 		rows++;
 		clistp = clistp->next;
 	}/* end for */
-	
+
+ERROR_TRAP:
+	nt_board_release_ref(h_board);
 	return result_state; 
 }
 
-static ctx_threadlist_tp init_context(nt_2ch_selected_item_tp selectp)
+static ctx_threadlist_tp init_context(nt_2ch_selected_item_handle h_select)
 {
 	ctx_threadlist_tp ctxp;
-	nt_board_tp boardp;
+	nt_board_handle h_board;
+	nt_thread_handle h_thread;
+	nt_enum_handle h_enum_thread;
+	nt_link_tp linkp;
+	thread_disp_data_tp datap;
+	int seq_no;
+	int num_res;
+	const wchar_t *title;
+	const wchar_t *dat_name;
+	
+	
 	ctxp = (ctx_threadlist_tp)calloc(1,sizeof(ctx_threadlist_t));
 
 	if(!ctxp)
 		return NULL;
 
-	boardp = selectp->selected_boardp;
-	ctxp->thread_num = nt_link_num(boardp->threadlistp);
+	h_board = nt_get_selected_board(h_select);
+	assert(h_board);
+	ctxp->thread_num = nt_board_get_thread_count(h_board);
 	ctxp->tracked_thread_num = ctxp->thread_num;
 
 	ctxp->regex_init = FALSE;
 	ctxp->sel_thread_no = -1;
+	ctxp->prev_state = DISP_STATE_BOARDMENU;
 
 	ctxp->thread_data_list = NULL;
-	ctxp->thread_disp_list = 
-		nt_link_copy(boardp->threadlistp);
-	if(!ctxp->thread_disp_list){
+	h_enum_thread = nt_board_get_thread_enum(h_board);
+	nt_board_release_ref(h_board);
+	if(!h_enum_thread){
 		free(ctxp);
 		return NULL;
 	}
+	while(NULL != (h_thread = (nt_thread_handle)nt_enum_fetch(h_enum_thread))){
+		seq_no = nt_thread_get_seq_number(h_thread);
+		title = nt_thread_get_title(h_thread);
+		dat_name = nt_thread_get_dat_name(h_thread);
+		num_res = nt_thread_get_res_count(h_thread);
+		datap = disp_data_alloc(seq_no, num_res, title, dat_name);
+		if(!datap){
+			if(ctxp->thread_disp_list)
+				nt_all_link_free(ctxp->thread_disp_list, disp_data_free);
+			free(ctxp);
+			nt_enum_unset(h_enum_thread);
+			return NULL;
+		}
+		linkp = nt_link_add_data(ctxp->thread_disp_list, datap);
+		if(!ctxp->thread_disp_list)
+			ctxp->thread_disp_list = linkp;
+	}
 	ctxp->sort_type = NT_CMD_SORT_NUMBER;
+	nt_enum_unset(h_enum_thread);
 	return ctxp;
 }
 
-int merge_untracked_thread_log(nt_board_tp boardp,
-		nt_link_tp *disp_thread_listpp, nt_link_tp log_list)
+
+static int merge_untracked_thread_log(int init_seq_no, nt_board_handle h_board,
+		nt_link_tp disp_thread_listp, nt_link_tp log_list)
 {
-	wchar_t *title, *dat_name;
+	thread_disp_data_tp datap;
+	wchar_t *title;
+	const wchar_t *dat_name;//, *board_name;
 	wchar_t buf[512];
 	nt_usr_db_thread_data_tp log_datap;
-	nt_link_tp log_linkp = log_list;
+	nt_link_tp log_linkp, linkp;
+	int seq_no;
+	
+	assert(disp_thread_listp);
+	
+	//board_name = nt_board_get_name(h_board);
+	seq_no = init_seq_no;
+	log_linkp = log_list;
 	do{
 		log_datap =
 			(nt_usr_db_thread_data_tp)log_linkp->data;
 		dat_name = log_datap->dat_name;
-		if(nt_get_thread_by_dat_name(
-				*disp_thread_listpp, dat_name)){
+		if(get_thread_data_by_dat_name(dat_name, disp_thread_listp)){
+			;
 		}else if(!(title = nt_read_thread_title(
-				boardp, dat_name))){
+				h_board, dat_name))){
+			;
 		}else{
 			wcscpy(buf, NT_TEXT_UNTRACKED_LOG_NAME);
 			wcscat(buf, title);
-			nt_thread_alloc(disp_thread_listpp,
-					buf, dat_name, log_datap->read_count);
 			free(title);
+			datap = disp_data_alloc(
+				seq_no, log_datap->read_count, buf, dat_name);
+			if(datap){
+				linkp = nt_link_add_data(disp_thread_listp, datap);
+				assert(linkp);
+				seq_no++;
+			}
 		}
 		log_linkp = log_linkp->next;
 	}while(log_linkp != log_list);
-	return nt_link_num(*disp_thread_listpp);
+	return nt_link_num(disp_thread_listp);
+}
+
+static thread_disp_data_tp disp_data_alloc(
+		int seq_no, int num_res, const wchar_t *title, const wchar_t *dat_name)
+{
+	thread_disp_data_tp datap;
+	datap = malloc(sizeof(thread_disp_data_t));
+	if(!datap)
+		return NULL;
+	datap->seq_no = seq_no;
+	datap->num_read = 0;
+	datap->num_res = num_res;
+	datap->title = nt_w_str_clone(title);
+	datap->dat_name = nt_w_str_clone(dat_name);
+	return datap;
+}
+
+static void disp_data_free(void *ptr)
+{
+	thread_disp_data_tp datap;
+	datap = (thread_disp_data_tp)ptr;
+	free(datap->dat_name);
+	free(datap->title);
+	free(datap);
+}
+
+static BOOL get_thread_data_by_dat_name(const wchar_t *dat_name, nt_link_tp disp_list)
+{
+	thread_disp_data_tp datap;
+	nt_link_tp linkp;
+	
+	linkp = disp_list;
+	do{
+		datap = (thread_disp_data_tp)linkp->data;
+		if(0 == wcscmp(dat_name, datap->dat_name))
+			return TRUE;
+		linkp = linkp->next;
+	}while(linkp != disp_list);
+	return FALSE;
 }
 
 
@@ -504,10 +634,10 @@ void init_threadlist_ctx(void *ptr)
 
 void free_threadlist_ctx(void *ptr)
 {
-	nt_link_tp linkp, nextp;
-	nt_thread_tp threadp;
-	int num;
-	wchar_t *cptr;
+	//nt_link_tp linkp, nextp;
+	//nt_thread_handle h_thread;
+	//int num;
+	//wchar_t *cptr;
 
 	ctx_threadlist_tp ctxp;
 	if(!ptr)
@@ -518,7 +648,8 @@ void free_threadlist_ctx(void *ptr)
 			nt_usr_db_thread_data_free);
 	}
 	if(ctxp->thread_disp_list){
-		linkp = ctxp->thread_disp_list->next;
+		nt_all_link_free(ctxp->thread_disp_list, disp_data_free);
+		/*linkp = ctxp->thread_disp_list->next;
 		num = 1;
 		while(linkp != ctxp->thread_disp_list){
 			nextp = linkp->next;
@@ -534,7 +665,7 @@ void free_threadlist_ctx(void *ptr)
 			num++;
 			linkp = nextp;
 		}
-		free(linkp);
+		free(linkp);*/
 	}
 	if(ctxp->regex_init){
 		regfree(&(ctxp->regex));
@@ -547,7 +678,7 @@ static BOOL search_line_asc(regex_t *regexp,
 				nt_link_tp threadlistp, int *sel_thread_no)
 {
 	wchar_t *cptr;
-	nt_thread_tp threadp;
+	thread_disp_data_tp datap;
 	nt_link_tp clistp;
 	size_t nmatch = 5;
 	regmatch_t pmatch[5];
@@ -558,8 +689,8 @@ static BOOL search_line_asc(regex_t *regexp,
 	cur = 0;
 	do{
 		if(cur > *sel_thread_no){
-			threadp = (nt_thread_tp)clistp->data;
-			cptr = threadp->name;
+			datap = (thread_disp_data_tp)clistp->data;
+			cptr = datap->title;
 			if(0 < wcstombs(buf, cptr, sizeof(buf))){
 			    if(0 == regexec(regexp, buf,
 					    nmatch, pmatch, 0)){
@@ -577,8 +708,8 @@ static BOOL search_line_asc(regex_t *regexp,
 		if(cur > *sel_thread_no)
 			break;
 
-		threadp = (nt_thread_tp)clistp->data;
-		cptr = threadp->name;
+		datap = (thread_disp_data_tp)clistp->data;
+		cptr = datap->title;
 		if(0 < wcstombs(buf, cptr, sizeof(buf))){
 		    if(0 == regexec(regexp, buf,
 				    nmatch, pmatch, 0)){
@@ -595,7 +726,7 @@ static BOOL search_line_desc(regex_t *regexp,
 				nt_link_tp threadlistp, int *sel_thread_no)
 {
 	wchar_t *cptr;
-	nt_thread_tp threadp;
+	thread_disp_data_tp datap;
 	nt_link_tp clistp;
 	size_t nmatch = 5;
 	regmatch_t pmatch[5];
@@ -608,8 +739,8 @@ static BOOL search_line_desc(regex_t *regexp,
 	cur = num - 1;
 	do{
 		if(cur < *sel_thread_no){
-			threadp = (nt_thread_tp)clistp->data;
-			cptr = threadp->name;
+			datap = (thread_disp_data_tp)clistp->data;
+			cptr = datap->title;
 			if(0 < wcstombs(buf, cptr, sizeof(buf))){
 			    if(0 == regexec(regexp, buf,
 					    nmatch, pmatch, 0)){
@@ -627,8 +758,8 @@ static BOOL search_line_desc(regex_t *regexp,
 		if(cur < *sel_thread_no)
 			break;
 
-		threadp = (nt_thread_tp)clistp->data;
-		cptr = threadp->name;
+		datap = (thread_disp_data_tp)clistp->data;
+		cptr = datap->title;
 		if(0 < wcstombs(buf, cptr, sizeof(buf))){
 		    if(0 == regexec(regexp, buf,
 				    nmatch, pmatch, 0)){
@@ -642,17 +773,35 @@ static BOOL search_line_desc(regex_t *regexp,
 	return FALSE;
 }
 
+static thread_disp_data_tp get_thread_by_seq_no(nt_link_tp linkp, int seq_no)
+{
+	nt_link_tp workp;
+	thread_disp_data_tp datap;
+	int n;
+
+	workp = linkp;
+	do{
+		datap = (thread_disp_data_tp)workp->data;
+		n = datap->seq_no;
+		if(n == seq_no)
+			return datap;
+		workp = workp->next;
+	}while(workp != linkp);
+	return NULL;
+}
+
 static int get_thread_index_by_seqno(nt_link_tp linkp, int seq_no)
 {
-	int idx;
+	int idx, n;
 	nt_link_tp workp;
-	nt_thread_tp threadp;
+	thread_disp_data_tp datap;
 
 	workp = linkp;
 	idx = 0;
 	do{
-		threadp = (nt_thread_tp)workp->data;
-		if(threadp->seq_no == seq_no)
+		datap = (thread_disp_data_tp)workp->data;
+		n = datap->seq_no;
+		if(n == seq_no)
 			return idx;
 		workp = workp->next;
 		idx++;
@@ -715,9 +864,9 @@ static int parse_cmd1(const char *param)
 static int sort_by_number(void *lhs, void *rhs)
 {
 	int num1, num2;
-	nt_thread_tp lthread, rthread;
-	lthread = (nt_thread_tp)lhs;
-	rthread = (nt_thread_tp)rhs;
+	thread_disp_data_tp lthread, rthread;
+	lthread = (thread_disp_data_tp)lhs;
+	rthread = (thread_disp_data_tp)rhs;
 	num1 = lthread->seq_no;
 	num2 = rthread->seq_no;
 
@@ -733,9 +882,9 @@ static int sort_by_unread(void *lhs, void *rhs)
 {
 	int num1, num2;
 	int unread1, unread2;
-	nt_thread_tp lthread, rthread;
-	lthread = (nt_thread_tp)lhs;
-	rthread = (nt_thread_tp)rhs;
+	thread_disp_data_tp lthread, rthread;
+	lthread = (thread_disp_data_tp)lhs;
+	rthread = (thread_disp_data_tp)rhs;
 
 	if(lthread->seq_no > g_ctxp->tracked_thread_num &&
 		rthread->seq_no <= g_ctxp->tracked_thread_num){
@@ -745,9 +894,9 @@ static int sort_by_unread(void *lhs, void *rhs)
 		return 1;
 	}
 	num1 = nt_usr_db_get_read_count_by_dat_name(
-		g_ctxp->thread_data_list, lthread->file_name);
+		g_ctxp->thread_data_list, lthread->dat_name);
 	num2 = nt_usr_db_get_read_count_by_dat_name(
-		g_ctxp->thread_data_list, rthread->file_name);
+		g_ctxp->thread_data_list, rthread->dat_name);
 	unread1 = -1;
 	unread2 = -1;
 	if(num1 > 0)
@@ -763,9 +912,9 @@ static int sort_by_unread(void *lhs, void *rhs)
 static int sort_by_read(void *lhs, void *rhs)
 {
 	int num1, num2;
-	nt_thread_tp lthread, rthread;
-	lthread = (nt_thread_tp)lhs;
-	rthread = (nt_thread_tp)rhs;
+	thread_disp_data_tp lthread, rthread;
+	lthread = (thread_disp_data_tp)lhs;
+	rthread = (thread_disp_data_tp)rhs;
 	if(lthread->seq_no > g_ctxp->tracked_thread_num &&
 		rthread->seq_no <= g_ctxp->tracked_thread_num){
 		return -1;
@@ -774,9 +923,9 @@ static int sort_by_read(void *lhs, void *rhs)
 		return 1;
 	}
 	num1 = nt_usr_db_get_read_count_by_dat_name(
-		g_ctxp->thread_data_list, lthread->file_name);
+		g_ctxp->thread_data_list, lthread->dat_name);
 	num2 = nt_usr_db_get_read_count_by_dat_name(
-		g_ctxp->thread_data_list, rthread->file_name);
+		g_ctxp->thread_data_list, rthread->dat_name);
 
 	if(num1 == num2)
 		return 0;
