@@ -24,6 +24,7 @@
 #include <iconv.h>
 
 #include "usr/favorite_t.h"
+#include "usr/usr_db_t.h"
 #include "utils/text.h"
 #include "utils/nt_conv_char.h" 
 
@@ -72,8 +73,14 @@ typedef struct tag_nt_favorite_thread_t {
 	wchar_t *dat_name;
 	wchar_t *board_name;
 	wchar_t *title;
+	int num_read;
+	int num_res;
 } nt_favorite_thread_t;
 
+static int local_wcscmp(void *lhs, void *rhs)
+{
+	return wcscmp((const wchar_t *)lhs, (const wchar_t *)rhs);
+}
 static void free_favorite_board(void *ptr);
 static void free_favorite_grp(void *ptr);
 static void free_grp_content(void *ptr);
@@ -143,6 +150,49 @@ const wchar_t* nt_favorite_thread_get_title(
 	return threadp->title;
 }
 
+int nt_favorite_thread_get_read_count(
+		nt_favorite_thread_handle handle)
+{
+	nt_favorite_thread_tp threadp;
+	threadp = (nt_favorite_thread_tp)handle;
+	assert(threadp);
+	assert(threadp->handle.chk_sum ==
+		NT_FAVORITE_THREAD_CHK_SUM);
+	return threadp->num_read;
+}
+
+int nt_favorite_thread_get_estimate_res_count(
+		nt_favorite_thread_handle handle)
+{
+	nt_favorite_thread_tp threadp;
+	threadp = (nt_favorite_thread_tp)handle;
+	assert(threadp);
+	assert(threadp->handle.chk_sum ==
+		NT_FAVORITE_THREAD_CHK_SUM);
+	return threadp->num_res;
+}
+void nt_favorite_thread_set_read_count(
+		nt_favorite_thread_handle handle, int read_count)
+{
+	nt_favorite_thread_tp threadp;
+	threadp = (nt_favorite_thread_tp)handle;
+	assert(threadp);
+	assert(threadp->handle.chk_sum ==
+		NT_FAVORITE_THREAD_CHK_SUM);
+	threadp->num_read = read_count;
+}
+
+void nt_favorite_thread_set_estimate_res_count(
+		nt_favorite_thread_handle handle, int res_count)
+{
+	nt_favorite_thread_tp threadp;
+	threadp = (nt_favorite_thread_tp)handle;
+	assert(threadp);
+	assert(threadp->handle.chk_sum ==
+		NT_FAVORITE_THREAD_CHK_SUM);
+	threadp->num_res = res_count;
+}
+
 BOOL nt_favorite_grp_get_flag(
 		nt_favorite_grp_handle handle, int flag)
 {
@@ -467,6 +517,8 @@ nt_favorite_thread_handle nt_favorite_thread_alloc(
 	threadp->title = nt_w_str_clone(title);
 	threadp->parent = grpp;
 	threadp->ref_count = 2;
+	threadp->num_read = 0;
+	threadp->num_res = 0;
 	
 	h_mutex = nt_favorite_thread_get_mutex(
 			(nt_favorite_thread_handle)&threadp->handle);
@@ -988,6 +1040,151 @@ BOOL nt_favorite_load_threads(nt_favorite_handle handle,
 	return TRUE;
 }
 
+static void read_count_data_free(void *ptr)
+{
+	nt_link_tp linkp;
+	assert(ptr);
+	linkp = (nt_link_tp)ptr;
+	nt_all_link_free(linkp, nt_usr_db_thread_data_free);
+}
+
+BOOL nt_favorite_set_num_res(
+		nt_2ch_model_handle h_model, nt_favorite_handle h_favorite)
+{
+	nt_favorite_grp_handle h_grp;
+	nt_favorite_thread_handle h_thread;
+	nt_thread_handle h_mthread;
+	nt_mutex_handle h_f_mutex, h_m_mutex;
+	nt_enum_handle h_enum_grp;
+	nt_enum_handle h_enum_thread;
+	const wchar_t *board_name, *dat_name;
+	int num_res;
+	
+	h_f_mutex = nt_favorite_get_mutex(h_favorite);
+	if(!h_f_mutex)
+		return FALSE;
+	h_m_mutex = nt_2ch_model_get_mutex(h_model);
+	if(!h_m_mutex)
+		return FALSE;
+	if(!nt_mutex_lock(h_f_mutex)){
+		return FALSE;
+	}
+	if(!nt_mutex_lock(h_m_mutex)){
+		nt_mutex_unlock(h_f_mutex);
+		return FALSE;
+	}
+	
+	h_enum_grp = nt_favorite_acquire_grp_enum(h_favorite);
+	if(h_enum_grp){
+		while(NULL != (h_grp =
+				(nt_favorite_grp_handle)nt_enum_fetch(h_enum_grp))){
+			h_enum_thread = nt_favorite_acquire_thread_enum(h_grp);
+			if(!h_enum_thread)
+				continue;
+			while(NULL != (h_thread =
+					(nt_favorite_thread_handle)nt_enum_fetch(h_enum_thread))){
+				board_name = nt_favorite_thread_get_board_name(h_thread);
+				if(!board_name)
+					continue;
+				dat_name = nt_favorite_thread_get_dat_name(h_thread);
+				if(!dat_name)
+					continue;
+    			h_mthread = nt_get_thread_by_board_and_dat_name(
+    						h_model, board_name, dat_name);
+    			if(!h_mthread)
+    				continue;
+				num_res = nt_thread_get_res_count(h_mthread);
+				nt_thread_release_ref(h_mthread);
+    			if(num_res == -1)
+    				continue;
+    			nt_favorite_thread_set_estimate_res_count(h_thread, num_res);
+			}
+			nt_enum_unset(h_enum_thread);
+		}
+		nt_enum_unset(h_enum_grp);
+	}
+	
+	if(!nt_mutex_unlock(h_m_mutex)){
+		assert(0);
+	}
+	if(!nt_mutex_unlock(h_f_mutex)){
+		assert(0);
+	}
+	return TRUE;
+}
+
+
+BOOL nt_favorite_set_read_count(nt_favorite_handle h_favorite, nt_usr_db_handle h_usr_db)
+{
+	nt_favorite_grp_handle h_grp;
+	nt_favorite_thread_handle h_thread;
+	nt_mutex_handle h_mutex;
+	nt_enum_handle h_enum_grp;
+	nt_enum_handle h_enum_thread;
+	nt_map_handle h_map;
+	const wchar_t *board_name, *dat_name;
+	nt_link_tp linkp;
+	void *ptr;
+	int read_count;
+	
+	h_mutex = nt_favorite_get_mutex(h_favorite);
+	if(!h_mutex)
+		return FALSE;
+	if(!nt_mutex_lock(h_mutex)){
+		return FALSE;
+	}
+	h_map = nt_map_alloc();
+	if(!h_map){
+		nt_mutex_unlock(h_mutex);
+		return FALSE;
+	}
+	
+	h_enum_grp = nt_favorite_acquire_grp_enum(h_favorite);
+	if(h_enum_grp){
+		while(NULL != (h_grp =
+				(nt_favorite_grp_handle)nt_enum_fetch(h_enum_grp))){
+			h_enum_thread = nt_favorite_acquire_thread_enum(h_grp);
+			if(!h_enum_thread)
+				continue;
+			while(NULL != (h_thread =
+					(nt_favorite_thread_handle)nt_enum_fetch(h_enum_thread))){
+				board_name = nt_favorite_thread_get_board_name(h_thread);
+				if(!board_name)
+					continue;
+				dat_name = nt_favorite_thread_get_dat_name(h_thread);
+				if(!dat_name)
+					continue;
+				ptr = nt_map_find(h_map, board_name);
+				if(ptr){
+					linkp = (nt_link_tp)ptr;
+				}else{
+					linkp = nt_usr_db_query_read_count_list(
+							h_usr_db, board_name);
+					if(!linkp)
+						continue;
+					if(!nt_map_add_pair(h_map, board_name, linkp))
+						continue;
+				}
+				read_count = nt_usr_db_get_read_count_by_dat_name(
+    					linkp, dat_name);
+    			if(read_count == -1)
+    				continue;
+    			
+    			nt_favorite_thread_set_read_count(h_thread, read_count);
+			}
+			nt_enum_unset(h_enum_thread);
+		}
+		nt_enum_unset(h_enum_grp);
+	}
+	
+	nt_map_free(h_map, read_count_data_free);
+	
+	if(!nt_mutex_unlock(h_mutex)){
+		assert(0);
+	}
+	return TRUE;
+}
+
 
 
 BOOL nt_favorite_retrieve_threads(nt_favorite_handle handle,
@@ -1028,7 +1225,7 @@ BOOL nt_favorite_retrieve_threads(nt_favorite_handle handle,
 			num = nt_enum_get_count(h_enum_thread);
 			assert(num >= 0);
 			open = nt_favorite_grp_get_flag(h_grp, NT_FAVORITE_GRP_FLAG_FOLDER_OPEN);
-			if(-1 == swprintf(wc, sizeof(wc)-1, 
+			if(-1 == swprintf(wc, sizeof(wc)/sizeof(wchar_t)-1, 
 					L"%ls,%d,%ls", cptr, num,  open ? L"open" : L" ")){
 				nt_enum_unset(h_enum_thread);
 				continue;
@@ -1139,6 +1336,77 @@ static BOOL nt_favorite_retrieve_thread(
 	return TRUE;
 }
 
+nt_link_tp nt_favorite_get_update_board_list(
+		nt_2ch_model_handle h_model, nt_favorite_handle h_favorite)
+{
+	nt_enum_handle h_enum_grp, h_enum_thread;
+	nt_favorite_grp_handle h_grp;
+	nt_favorite_thread_handle h_fthread;
+	nt_2ch_selected_item_handle h_select;
+	nt_category_handle h_category;
+	nt_board_handle h_board;
+	nt_mutex_handle h_mutex;
+	nt_link_tp res_linkp, tmp_linkp, board_name_linkp;
+	const wchar_t *board_name, *cptr;
+	
+	h_mutex = nt_favorite_get_mutex(h_favorite);
+	if(!h_mutex){
+		return NULL;
+	}
+	if(!nt_mutex_lock(h_mutex)){
+		return NULL;
+	}
+	h_enum_grp = nt_favorite_acquire_grp_enum(h_favorite);
+	if(!h_enum_grp){
+		nt_mutex_unlock(h_mutex);
+		return NULL;
+	}
+	res_linkp = NULL;
+	board_name_linkp = NULL;
+	while(NULL != (h_grp =
+			(nt_favorite_grp_handle)nt_enum_fetch(h_enum_grp))){
+		h_enum_thread = nt_favorite_acquire_thread_enum(h_grp);
+		if(!h_enum_thread)
+			continue;
+		while(NULL != (h_fthread =
+				(nt_favorite_thread_handle)nt_enum_fetch(h_enum_thread))){
+			board_name = nt_favorite_thread_get_board_name(h_fthread);
+			if(!board_name)
+				continue;
+			if(board_name_linkp){
+				cptr = (const wchar_t*)nt_link_find(
+						board_name_linkp, (void*)board_name, local_wcscmp);
+				if(cptr)
+					continue;
+			}
+			//fwprintf(stderr, L"%ls\n", board_name);
+			h_board = nt_get_board_by_name(h_model, board_name, &h_category);
+			if(h_board && h_category){
+				h_select = nt_2ch_selected_item_alloc();
+				if(h_select){
+					nt_set_selected_board(h_select, h_category, h_board);
+					tmp_linkp = nt_link_add_data(res_linkp, h_select);
+					if(tmp_linkp && !res_linkp)
+						res_linkp = tmp_linkp;
+					tmp_linkp = nt_link_add_data(board_name_linkp, (void*)board_name);
+					if(tmp_linkp && !board_name_linkp)
+						board_name_linkp = tmp_linkp;
+					
+				}
+				nt_category_release_ref(h_category);
+				nt_board_release_ref(h_board);
+			}
+		}
+		nt_enum_unset(h_enum_thread);
+	}
+	if(board_name_linkp)
+		nt_all_link_free(board_name_linkp, NULL);
+	nt_enum_unset(h_enum_grp);
+	if(!nt_mutex_unlock(h_mutex)){
+		assert(0);
+	}
+	return res_linkp;
+}
 
 void favorite_dump(nt_favorite_handle handle)
 {
@@ -1180,7 +1448,7 @@ void favorite_dump(nt_favorite_handle handle)
 		num = 1;
 		do{
 			boardp = linkp->data;
-			swprintf(wc, sizeof(wc), 
+			swprintf(wc, sizeof(wc)/sizeof(wchar_t), 
 				L"\t(%d) board name \"%ls\"\n", num, boardp->board_name);
 			nt_conv_wc2local(icd, wc, c, sizeof(c));
 			fputs(c, stderr);
@@ -1202,7 +1470,7 @@ void favorite_dump(nt_favorite_handle handle)
 				link2p = grpp->contents;
 				do{
 					threadp = link2p->data;
-					swprintf(wc, sizeof(wc),
+					swprintf(wc, sizeof(wc)/sizeof(wchar_t),
 						L"\t(%d) %ls[%ls] -- %ls\n", 
 						num, threadp->title, threadp->board_name, threadp->dat_name);
 					nt_conv_wc2local(icd, wc, c, sizeof(c));
